///File : strWxStdConv.hpp
//------------------------------------------------------------------------------
/*
    This file is part of DevisMaker.

    DevisMaker is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DevisMaker is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DevisMaker.  If not, see <http://www.gnu.org/licenses/>.
*/
//------------------------------------------------------------------------------
#include "wx/wxprec.h"

#ifdef __BORLANDC__
    #pragma hdrstop
#endif

#ifndef WX_PRECOMP
    #include "wx/wx.h"
#endif
//------------------------------------------------------------------------------
#include "math.h"
#include "config.hpp"
#include "buildVersionOptions.hpp"
#include "ConfigManager.hpp"
//------------------------------------------------------------------------------

#if not defined PACKAGE
    #error "This program should be built with its package !"// #define PACKAGE devismaker
#endif

#if not defined STR_WX_STD_CONV_HPP_DEVIS_MAKER
#define STR_WX_STD_CONV_HPP_DEVIS_MAKER
//------------------------------------------------------------------------------
struct StructWxColorFont
{
    wxFont wxfont;
    wxColour wxcolor;
};
//------------------------------------------------------------------------------
// String
std::string StrWxToStd(const wxString &str);
wxString StrStdToWx(const std::string &str);
wxString wxStrAutoCut(wxString str, const int &maxLen = 60);

// Filename (Pour Windows)
std::string filenameWxToStd(const wxString &str);
wxString filenameStdToWx(const std::string &str);

// Font
wxFont stdFontToWxFont(const StructStdFont &font);
StructStdFont wxFontToStdFont(const wxFont &font);

// Color
wxColour stdColorToWxColor(const StructStdColor &color);
StructStdColor wxColorToStdColor(const wxColour &color);

// ColorFont
StructWxColorFont stdColorFontToWxColorFont(const StructStdColorFont &colorFont);
StructStdColorFont wxColorFontToStdColorFont(const StructWxColorFont &colorFont);

//------------------------------------------------------------------------------
template < typename T >
bool wxStrToNb(wxString str, T *nb) // Avec verification
{
    // 1 : Remplacement des "." par des ","
    int lang = wxLocale::GetSystemLanguage();

    if(lang == wxLANGUAGE_FRENCH || lang == wxLANGUAGE_FRENCH_BELGIAN || lang == wxLANGUAGE_FRENCH_CANADIAN || lang == wxLANGUAGE_FRENCH_LUXEMBOURG || lang == wxLANGUAGE_FRENCH_MONACO || lang == wxLANGUAGE_FRENCH_SWISS)
    {
        // Si on est en france ==> On remplace les "." du pavé numérique par des ",".
        str.Replace(_T("."), _T(","));
    }

    // On transforme en double ==> Le plus de précision.
    double tempNb;
    bool isOk = str.ToDouble(&tempNb);

    // On cast en "T"

    *nb = T(tempNb);

	return isOk;
}
//------------------------------------------------------------------------------
// Sans verif...
template < typename T >
T wxStrToNb(const wxString &str)
{
    T tempNb;
    wxStrToNb(str, &tempNb);
    return tempNb;
}
//------------------------------------------------------------------------------
template < typename T >
wxString wxNbToStr(const T &nb)
{
    // On doit passer par les std::string pour eviter les nb scientifiques.
	return StrStdToWx(nbToString(nb));
}
//------------------------------------------------------------------------------
/*template < typename T >
wxString ArrangeNb(const T &nb, const int &synifNb = 2, const bool &addZeros = true, const bool &addSpaces = true)
{
    // 1 - Quel est le symole de la virgule ?
    wxChar virguleChar = '.'; // Tjr le "."

    // 2 - On remplie la wxString
    wxString tempStr = wxNbToStr(nb);

    // 3 - On arrondi
    if(tempStr.Len() - tempStr.Find(virguleChar) > synifNb + 1) // Trop de chiffres significatifs
    {
        wxChar lastNbChar = tempStr[tempStr.Find(virguleChar) + synifNb + 1];
        // ==> A faire
        if(wxStrToNb<int>(lastNbChar) >= 5) // Si le chiffre avant le dernier chiffre significatif est sup à 5 ==> rajout de 10 ^ -synifNb
        {
            // Utilise le nombre arrondi...
            tempStr = wxNbToStr(nb + pow(10, -synifNb));

        }
        tempStr = tempStr.Mid(0, tempStr.Find(virguleChar) + synifNb + 1);
    }

    // 3 - On ajoute les nbs significatifs
    if(addZeros)
    {
        if(tempStr.Find(virguleChar) == wxNOT_FOUND) // Pas de virgule
        {
            tempStr << virguleChar; // On la rajoute
        }

        while(tempStr.Len() - tempStr.Find(virguleChar) <= synifNb) // Une virgule mais pas assez de chiffre significatif
        {
            tempStr << _T("0");
        }
    }

    // Separation des "1000" (comme 1 123 899.21)
    if(addSpaces)
    {
        int pos = tempStr.Find(virguleChar);
        if(pos == wxNOT_FOUND)
        {
            pos = tempStr.Len() - 1;
        }

        pos -= 3; // On revient de 3.

        while(pos > 0)
        {
            tempStr.insert(pos, _T(" "));
            pos -= 3;
        }
    }

    return tempStr;
}*/
//------------------------------------------------------------------------------

// V2
template < typename T >
wxString ArrangePrice(const T &nb, const ConfigManager *cfg)
{
    const int synifNb = cfg->GetInt("nb_synifNb");

    // On remplie la wxString
    wxString tempStr = wxNbToStr(nb);

    // Quel est le symole de la virgule ?
    wxString virguleChar = StrStdToWx(cfg->GetString("nb_virguleString"));
    tempStr.Replace( _T("."), virguleChar);

    // On arrondi
    int virgulePos = tempStr.Find(virguleChar);
    if(virgulePos != wxNOT_FOUND && cfg->GetBool("nb_rondIfTooMuchSynifNb") && tempStr.Len() - virgulePos > synifNb + 1) // Trop de chiffres significatifs
    {
        wxChar lastNbChar = tempStr[tempStr.Find(virguleChar) + synifNb + 1];
        // ==> A faire
        if(wxStrToNb<int>(lastNbChar) >= 5) // Si le chiffre avant le dernier chiffre significatif est sup à 5 ==> rajout de 10 ^ -synifNb
        {
            // Utilise le nombre arrondi...
            tempStr = wxNbToStr(nb + pow(10, -synifNb));
            tempStr.Replace( _T("."), virguleChar);
        }

        tempStr = tempStr.Mid(0, tempStr.Find(virguleChar) + synifNb + 1);
    }

    // 3 - On ajoute les nbs significatifs
    if(cfg->GetBool("nb_addZeros"))
    {
        if(tempStr.Find(virguleChar) == wxNOT_FOUND) // Pas de virgule
        {
            tempStr << virguleChar; // On la rajoute
        }

        while(tempStr.Len() - tempStr.Find(virguleChar) <= synifNb) // Une virgule mais pas assez de chiffre significatif
        {
            tempStr << _T("0");
        }
    }

    // Separation des "1000" (comme 1 123 899.21)
    if(cfg->GetBool("nb_addSpaces"))
    {
        wxString spaceString = StrStdToWx(cfg->GetString("nb_spaceString"));

        int pos = tempStr.Find(virguleChar);
        if(pos == wxNOT_FOUND)
        {
            pos = tempStr.Len() - 1;
        }

        pos -= 3; // On revient de 3.

        while(pos > 0)
        {
            tempStr.insert(pos, spaceString);
            pos -= 3;
        }
    }

    tempStr = StrStdToWx(cfg->GetString("nb_before")) + tempStr + StrStdToWx(cfg->GetString("nb_after"));

    return tempStr;
}
//------------------------------------------------------------------------------


#endif // STR_WX_STD_CONV_HPP_DEVIS_MAKER
